﻿using IMES_Middleware_Platform.Api.Core.Consts;
using IMES_Middleware_Platform.Api.Domain.Module;
using IMES_Middleware_Platform.Api.Domain.Module.Dto;
using IMES_Middleware_Platform.Api.Services.Module.Input;
using IMES_Middleware_Platform.Api.Services.Module.Output;
using Microsoft.AspNetCore.Mvc;
using System.Threading.Tasks;
using ZhonTai;
using ZhonTai.Admin.Core.Dto;
using ZhonTai.Admin.Services;
using ZhonTai.DynamicApi;
using ZhonTai.DynamicApi.Attributes;

namespace IMES_Middleware_Platform.Api.Services.Module
{
    /// <summary>
    /// 模块服务
    /// </summary>
    [Order(1010)]
    [DynamicApi(Area = ApiConsts.AreaName)]
    public class ModuleService : BaseService, IModuleService, IDynamicApi
    {
        private IModuleRepository _moduleRepository => LazyGetRequiredService<IModuleRepository>();

        public ModuleService()
        {
        }

        /// <summary>
        /// 查询模块
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public async Task<ModuleGetOutput> GetAsync(long id)
        {
            var result = await _moduleRepository.GetAsync<ModuleGetOutput>(id);
            return result;
        }

        /// <summary>
        /// 查询分页
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<PageOutput<ModuleListOutput>> GetPageAsync(PageInput<ModuleGetPageDto> input)
        {
            var key = input.Filter?.Name;

            var list = await _moduleRepository.Select
            .WhereIf(key.NotNull(), a => a.Name.Contains(key))
            .Count(out var total)
            .OrderByDescending(true, c => c.Id)
            .Page(input.CurrentPage, input.PageSize)
            .ToListAsync<ModuleListOutput>();

            var data = new PageOutput<ModuleListOutput>()
            {
                List = list,
                Total = total
            };

            return data;
        }

        /// <summary>
        /// 新增
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<long> AddAsync(ModuleAddInput input)
        {
            var entity = Mapper.Map<ModuleEntity>(input);
            await _moduleRepository.InsertAsync(entity);

            return entity.Id;
        }

        /// <summary>
        /// 修改
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task UpdateAsync(ModuleUpdateInput input)
        {
            var entity = await _moduleRepository.GetAsync(input.Id);
            if (!(entity?.Id > 0))
            {
                throw ResultOutput.Exception("模块不存在");
            }

            Mapper.Map(input, entity);
            await _moduleRepository.UpdateAsync(entity);
        }

        /// <summary>
        /// 彻底删除
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public async Task DeleteAsync(long id)
        {
            await _moduleRepository.DeleteAsync(m => m.Id == id);
        }

        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public async Task SoftDeleteAsync(long id)
        {
            await _moduleRepository.SoftDeleteAsync(id);
        }

        /// <summary>
        /// 批量删除
        /// </summary>
        /// <param name="ids"></param>
        /// <returns></returns>
        public async Task BatchSoftDeleteAsync(long[] ids)
        {
            await _moduleRepository.SoftDeleteAsync(ids);
        }

    }
}